#!/usr/bin/env python
# coding: UTF-8

import sys
import os
import threading
import time
import datetime
from optparse import *

import pyqtgraph as pg
from pyqtgraph import QtGui, QtCore

sys.path.append('./MNLib/')
from apptag import AppTag

options = None
args = None

bExit = False
bUpdate = False
bEnableVolt = False
bEnableLQI = False
bEnableOutput = False
bEnableLog = False
bEnablePlayer = False

SampleNum = 128
YRange = 2
FPS = 100

RcvAddr = None

DataX = []
DataY = []
DataZ = []
DataLQI = []
DataVolt = []

class graph():
	def __init__(self):
		self.Window = pg.GraphicsWindow()
		self.Window.setWindowTitle("MONOWIRELESS AppTag Viewer")

		__GraphNum = 1
		__GraphNum += 1 if bEnableVolt else 0
		__GraphNum += 1 if bEnableLQI else 0

		self.Window.resize(800,300*__GraphNum)

		pg.setConfigOptions(antialias=True)

		self.plt = self.Window.addPlot()
		self.plt.showGrid( True, True, 1 )
		self.plt.setYRange(-1*YRange,YRange)
		if not bEnableVolt and not bEnableLQI:
			self.plt.setLabel('bottom','Sample')
		self.plt.setLabel('left','Accel','g')
		self.plt.addLegend()

		self.curve = [None]*3
		self.curve[0] = self.plt.plot(pen=(255,0,0), name='X')
		self.curve[1] = self.plt.plot(pen=(0,255,0), name='Y')
		self.curve[2] = self.plt.plot(pen=(0,255,255), name='Z')

		if bEnableVolt:
			self.Window.nextRow()
			self.pltvolt = self.Window.addPlot()
			self.pltvolt.setYRange(1950,3600)
			self.pltvolt.showGrid( True, True, 1 )
			self.pltvolt.setLabel('left','Power (mV)')
			if not bEnableLQI:
				self.pltvolt.setLabel('bottom','Sample')
			self.curvevolt = self.pltvolt.plot()

		if bEnableLQI:
			self.Window.nextRow()
			self.pltlqi = self.Window.addPlot()
			self.pltlqi.setYRange(0,255)
			self.pltlqi.showGrid( True, True, 1 )
			self.pltlqi.setLabel('left','LQI')
			self.pltlqi.setLabel('bottom','Sample')
			self.curvelqi = self.pltlqi.plot()

		self.timer = QtCore.QTimer()
		self.timer.timeout.connect(self.update)
		self.timer.start(0)

	def update(self):
		global bUpdate
		if bUpdate:
			if bEnablePlayer:
				self.Window.setWindowTitle("MONOWIRELESS App_Tag Viewer : " + RcvAddr)
			else:
				self.Window.setWindowTitle("MONOWIRELESS App_Tag Viewer : " + RcvAddr[1:])
			self.curve[0].setData(DataX)
			self.curve[1].setData(DataY)
			self.curve[2].setData(DataZ)

			if bEnableVolt:
				self.curvevolt.setData(DataVolt)

			if bEnableLQI:
				self.curvelqi.setData(DataLQI)

			bUpdate = False


def ParseArgs():
	global options, args

	parser = OptionParser()
	if os.name == 'nt':
		parser.add_option('-t', '--target', type='string', help='target for connection', dest='target', default='COM3')
	else:
		parser.add_option('-t', '--target', type='string', help='target for connection', dest='target', default='/dev/ttyUSB0')

	parser.add_option('-b', '--baud', dest='baud', type='int', help='baud rate for serial connection.', metavar='BAUD', default=115200)
	parser.add_option('-s', '--serialmode', dest='format', type='string', help='serial data format type. (Ascii or Binary)',  default='Ascii')
	parser.add_option('-n', '--num', dest='samplenum', type='int', help='plot sample number.',  default=128)
	parser.add_option('-r', '--range', dest='range', type='int', help='Y axis range',  default=2)
	parser.add_option('-a', '--autolog', dest='autolog', action='store_true', help='enable auto log.', default=False)
	parser.add_option('-v', '--volt', dest='volt', action='store_true', help='plot power voltage.', default=False)
	parser.add_option('-l', '--lqi', dest='lqi', action='store_true', help='plot LQI.', default=False)
	parser.add_option('-o', '--output', dest='output', action='store_true', help='output arrived data to console.', default=False)
	parser.add_option('-p', '--player', dest='play', action='store_true', help='execute log player', default=False)
	parser.add_option('-f', '--fps', dest='fps', type='int', help='set update frequency (fps) when log player.',  default=100)
	(options, args) = parser.parse_args()

def ReadSensor(Tag):
	global bUpdate, DataX, DataY, DataZ, DataVolt, DataLQI, RcvAddr
	while True:
		if Tag.ReadSensorData():
			Dic = Tag.GetDataDict()
			if Dic['Sensor'] == 0x35:
				if RcvAddr == None:
					RcvAddr = Dic['EndDeviceSID']
				elif RcvAddr != Dic['EndDeviceSID']:
					continue

				if Dic['Mode'] == 0xFA:
					if len(DataX) == SampleNum:
						DataX[0:] = DataX[10:]
					DataX += Dic['AccelerationX']
					if len(DataX) > SampleNum:
						DataX[0:] = DataX[len(DataX)-SampleNum:]

					if len(DataY) == SampleNum:
						DataY[0:] = DataY[10:]
					DataY += Dic['AccelerationY']
					if len(DataY) > SampleNum:
						DataY[0:] = DataY[len(DataY)-SampleNum:]

					if len(DataZ) == SampleNum:
						DataZ[0:] = DataZ[10:]
					DataZ += Dic['AccelerationZ']
					if len(DataZ) > SampleNum:
						DataZ[0:] = DataZ[len(DataZ)-SampleNum:]

					if len(DataVolt) == SampleNum:
						DataVolt[0:] = DataVolt[10:]
					i = 0
					while i < len(Dic['AccelerationX']):
						DataVolt.append(Dic['Power'])
						i += 1

					if len(DataVolt) > SampleNum:
						DataVolt[0:] = DataVolt[len(DataVolt)-SampleNum:]

					if len(DataLQI) == SampleNum:
						DataLQI[0:] = DataLQI[10:]
					i = 0
					while i < len(Dic['AccelerationX']):
						DataLQI.append(Dic['LQI'])
						i += 1

					if len(DataLQI) > SampleNum:
						DataLQI[0:] = DataLQI[len(DataLQI)-SampleNum:]

				else:
					if len(DataX) == SampleNum:
						DataX[0:] = DataX[1:]
						DataY[0:] = DataY[1:]
						DataZ[0:] = DataZ[1:]
						DataVolt[0:] = DataVolt[1:]
						DataLQI[0:] = DataLQI[1:]

					if Dic['Mode'] == 0xFB:
						DataX.append(Dic['Degree'])
#						DataY.append(0)
#						DataZ.append(0)
					else:
						DataX.append(Dic['AccelerationX'])
						DataY.append(Dic['AccelerationY'])
						DataZ.append(Dic['AccelerationZ'])
					DataVolt.append(Dic['Power'])
					DataLQI.append(Dic['LQI'])

				bUpdate = True
				if bEnableOutput:
					Tag.ShowSensorData()
				if bEnableLog:
					Tag.OutputData()

		if bExit:
			break

def ReadLog(logdata):
	global bUpdate, DataX, DataY, DataZ, DataVolt, DataLQI, RcvAddr

	loglist = list()
	tmplist = list()
	for log in logdata:
		log = str(log)
		log = log.strip()
		log = log.replace('\t','')
		datalist = log.split(',')
		if datalist[0] == 'ArriveTime':
			continue
		elif datalist[0] == '':
			tmpX = datalist[11]
			tmpY = datalist[12]
			tmpZ = datalist[13]
			datalist = tmplist
			datalist[11] = tmpX
			datalist[12] = tmpY
			datalist[13] = tmpZ

			datalist[0] += datetime.timedelta(milliseconds=10)
			tmplist = datalist[:]
		else:
			RcvAddr = datalist[2]
			datalist[0] = datetime.datetime.strptime( (datalist[0]+'000'), '%Y/%m/%d %H:%M:%S.%f' )
			tmplist = datalist[:]

		del datalist[1:4]
		del datalist[2:3]
		del datalist[3:7]
		loglist.append(datalist)

	i=0
	ttime = 0
	while i < len(loglist):

		DataLQI.append(int(loglist[i][1]))
		DataVolt.append(int(loglist[i][2]))

		DataX.append(float(loglist[i][3]))
		DataY.append(float(loglist[i][4]))
		DataZ.append(float(loglist[i][5]))

		DataLQI[0:] = DataLQI[1:]
		DataVolt[0:] = DataVolt[1:]
		DataX[0:] = DataX[1:]
		DataY[0:] = DataY[1:]
		DataZ[0:] = DataZ[1:]

		bUpdate = True

		if i+1 != len(loglist):
			stime = (loglist[i+1][0]-loglist[i][0]).total_seconds()
			time.sleep(1.0/float(FPS))

		i += 1

		if bExit:
			break

if __name__ == '__main__':
	ParseArgs()

	SampleNum = options.samplenum
	YRange = options.range
	bEnableVolt = options.volt
	bEnableLQI = options.lqi
	bEnableOutput = options.output
	bEnableLog = options.autolog
	bEnablePlayer = options.play
	FPS = options.fps

	DataX = [0] * SampleNum
	DataY = [0] * SampleNum
	DataZ = [0] * SampleNum
	DataVolt = [0] * SampleNum
	DataLQI = [0] * SampleNum

	if YRange < 0:
		YRange *= -1

	if bEnablePlayer:
		try:
			logfile = open( options.target, 'r' )
			logdata = logfile.readlines()
			logfile.close()
		except:
			print( "Please assign target file..." )
			exit(1)

		subThread = threading.Thread( target=ReadLog, args=(logdata,) )
	else:
		try:
			Tag = AppTag(port=options.target, baud=options.baud, tout=0.05, sformat=options.format)
		except:
			exit(1)
		subThread = threading.Thread( target=ReadSensor, args=(Tag,) )

	subThread.setDaemon(True)
	subThread.start()

	g = graph()
	if (sys.flags.interactive != 1) or not hasattr(QtCore, 'PYQT_VERSION'):
		QtGui.QApplication.instance().exec_()

	bExit = True
	subThread.join()

	if not bEnablePlayer:
		del Tag
